// ObserverSwarm.m
//--------------------------------------------------------------------

#import "ObserverSwarm.h"

//--------------------------------------------------------------------

@implementation ObserverSwarm

//--------------------------------------------------------------------

+ createBegin: aZone
{

   ObserverSwarm * obj;
   id <ProbeMap>   probeMap;

   TRACE0(printf("\n%8X - ObserverSwarm entry createBegin: %8X\n",
                 (int)self,(int)aZone);)

   obj = [super createBegin: aZone];

   obj->displayFrequency = 100;
   obj->zoomFactor=2;

   // Also, build a customized probe map. Without a probe map, the
   // default is to show all variables and messages. Here we choose to
   // customize the appearance of the probe, give a nicer interface.

   probeMap = [EmptyProbeMap createBegin: aZone];
   [probeMap setProbedClass: [self class]];
   probeMap = [probeMap createEnd];

   // Add in a bunch of variables, one per simulation parameters

   [probeMap addProbe: [probeLibrary getProbeForVariable:
    "displayFrequency" inClass: [self class]]];
   [probeMap addProbe: [probeLibrary getProbeForVariable:
    "zoomFactor" inClass: [self class]]];

   // Now install our custom probeMap into the probeLibrary.

   [probeLibrary setProbeMap: probeMap For: [self class]];

   TRACE0(printf("%8X - ObserverSwarm exit createBegin: %8X\n",
                 (int)self,(int)obj);)

   return obj;

}

//--------------------------------------------------------------------

- createEnd
{

   id obj;

   TRACE0(printf("\n%8X - ObserverSwarm entry createEnd\n",(int)self);)

   obj = [super createEnd];

   TRACE0(printf("%8X - ObserverSwarm exit createEnd: %8X\n",
                 (int)self,(int)obj);)

   return obj;

}

//--------------------------------------------------------------------

- buildObjects
{

   int i,j,k,l;

   TRACE0(printf("\n%8X - ObserverSwarm entry buildObjects\n",(int)self);)

   [super buildObjects];

   // First, we create the model that we're actually observing. The
   // model is a subswarm of the observer.
   // We create in the modelSwarm in "self" which creates a modelSwarm
   // in it's *own* Zone within the observerSwarm.

   modelSwarm = [ModelSwarm create: self];

   // Now create probe objects on the model and ourselves. This gives
   // a simple user interface to let the user change parameters.

   CREATE_PROBE_DISPLAY (modelSwarm);
   CREATE_PROBE_DISPLAY (self);

   // Instruct the control panel to wait for a button event.
   // We halt here until someone hits a control panel button.

   // Now that we're using Probes, the user can set the parameters
   // in the ModelSwarm probe window - we halt here to allow
   // the user to change parameters.

   [controlPanel setStateStopped];

   // When the user hits "go" on the control panel, we resume here
   // OK - the user said "go" so we're ready to start

   [modelSwarm buildObjects];

   antGraph = [EZGraph createBegin: self];
   [antGraph setTitle: "number of ants"];
   [antGraph setAxisLabelsX: "Cycles" Y: "Ants"];
   antGraph = [antGraph createEnd];

   [antGraph createSequence: "ants"
               withFeedFrom: [modelSwarm getAntList]
                andSelector: M(getCount)];
   [antGraph createSequence: "queens"
               withFeedFrom: modelSwarm
                andSelector: M(getNumberOfQueens)];
   [antGraph createSequence: "eatenAnts"
               withFeedFrom: modelSwarm
                andSelector: M(getEatenAnts)];
   // Now get down to building our own display objects.

   // First, create a colormap: this is a global resource, the
   // information here is used by lots of different objects.
   // To see the paths of the ants simply inhibit line 123 and
   // restore line 124

   colorMap = [Colormap create: self];

   [colorMap setColor: 0 ToName: "white"];
// [colorMap setColor: 1 ToName: "white"];
   [colorMap setColor: 1 ToName: "green"];
   [colorMap setColor: 2 ToName: "red"];
   [colorMap setColor: 3 ToName: "black"];

   // Next, create a 2d window for display,
   // set its size, zoom factor, title.

   worldRaster = [ZoomRaster create: self];
   [worldRaster setColormap: colorMap];
   [worldRaster setZoomFactor: zoomFactor];
   [worldRaster setWidth: [[modelSwarm getWorld] getSizeX]
                  Height: [[modelSwarm getWorld] getSizeY]];
   [worldRaster setWindowTitle: "Langton Ant Space"];
   [worldRaster pack];                          // draw the window.

   i = [[modelSwarm getWorld] getSizeX];
   j = [[modelSwarm getWorld] getSizeY];
   for(k=1;k<=i;k++) for(l=1;l<=j;l++)
       [worldRaster drawPointX: i Y:l Color: 0];

   // Now create a Value2dDisplay: this is a special object that will
   // display arbitrary 2d value arrays on a given Raster widget.

   traceDisplay = [Value2dDisplay createBegin: self];
   [traceDisplay setDisplayWidget: worldRaster colormap: colorMap];
   [traceDisplay setDiscrete2dToDisplay: [modelSwarm getTraces]];
   traceDisplay = [traceDisplay createEnd];

   // And also create an Object2dDisplay: this object draws agents on
   // the worldRaster widget for us.

   agentDisplay = [Object2dDisplay createBegin: self];
   [agentDisplay setDisplayWidget: worldRaster];
   [agentDisplay setDiscrete2dToDisplay: [modelSwarm getWorld]];
   [agentDisplay setObjectCollection: [modelSwarm getAntList]];
   [agentDisplay setDisplayMessage: M(drawSelfOn:)];
   agentDisplay = [agentDisplay createEnd];

   // Also, tell the world raster to send mouse clicks to the agentDisplay
   // this allows the user to right-click on the display to probe the agents.

   [worldRaster setButton: ButtonRight
                   Client: agentDisplay
                  Message: M(makeProbeAtX:Y:)];

   TRACE0(printf("%8X - ObserverSwarm exit buildObjects %8X\n",
                 (int)self,(int)self);)

   return self;

}

//--------------------------------------------------------------------

- buildActions
{

   int i;

   TRACE0(printf("\n%8X - ObserverSwarm entry buildActions\n",(int)self);)

   // Create the actions necessary for the simulation.

   [super buildActions];

   // First, let our model swarm build its own schedule.

   [modelSwarm buildActions];

   // Create an ActionGroup for display.

   displayActions = [ActionGroup create: self];
   controlActions = [ActionGroup create: self];

   // Schedule up the methods to draw the display of the world

   [displayActions createActionTo: traceDisplay message: M(display)];
   [displayActions createActionTo: agentDisplay message: M(display)];
   [displayActions createActionTo: worldRaster  message: M(drawSelf)];
   [displayActions createActionTo: antGraph  message: M(step)];
   //   [displayActions createActionTo: modelSwarm message: M(verify)];

   [controlActions createActionTo: actionCache  message: M(doTkEvents)];

   // And the display schedule. Note the repeat interval is set from our
   // own Swarm data structure. Display is frequently the slowest part
   // of a simulation, so redrawing less frequently can be a help.

   displaySchedule = [Schedule createBegin: self];
   [displaySchedule setRepeatInterval: displayFrequency];
   displaySchedule = [displaySchedule createEnd];
   [displaySchedule at: 0 createAction: displayActions];
   for(i=0;i<displayFrequency;i++)
   [displaySchedule at: i createAction: controlActions];

   TRACE0(printf("%8X - ObserverSwarm exit buildActions %8X\n",
                 (int)self,(int)self);)

   return self;

}

//--------------------------------------------------------------------

- activateIn: swarmContext
{
   // activateIn: - activate the schedules so they're ready to run.

   TRACE0(printf("\n%8X - ObserverSwarm entry activateIn: %8x\n",
                 (int)self,(int)swarmContext);)

   [super activateIn: swarmContext];

   // Activate the model swarm in ourselves. The model swarm is a
   // subswarm of the observer swarm.

   [modelSwarm activateIn: self];

   // Now activate our schedule in ourselves. This arranges for the
   // execution of the schedule we built.

   [displaySchedule activateIn: self];

   // Activate returns the swarm activity - the thing that's ready to run.

   TRACE0(printf("%8X - ObserverSwarm exit activateIn %8X\n",
                 (int)self,(int)[self getSwarmActivity]);)

   return [self getSwarmActivity];

}

//--------------------------------------------------------------------

@end
